import random
from mpmath import *
from mpmath.libmp import *


def test_basic_string():
    """
    Test basic string conversion
    """
    mp.dps = 15
    assert mpf('3') == mpf('3.0') == mpf('0003.') == mpf('0.03e2') == mpf(3.0)
    assert mpf('30') == mpf('30.0') == mpf('00030.') == mpf(30.0)
    for i in range(10):
        for j in range(10):
            assert mpf('%ie%i' % (i,j)) == i * 10**j
    assert str(mpf('25000.0')) == '25000.0'
    assert str(mpf('2500.0')) == '2500.0'
    assert str(mpf('250.0')) == '250.0'
    assert str(mpf('25.0')) == '25.0'
    assert str(mpf('2.5')) == '2.5'
    assert str(mpf('0.25')) == '0.25'
    assert str(mpf('0.025')) == '0.025'
    assert str(mpf('0.0025')) == '0.0025'
    assert str(mpf('0.00025')) == '0.00025'
    assert str(mpf('0.000025')) == '2.5e-5'
    assert str(mpf(0)) == '0.0'
    assert str(mpf('2.5e1000000000000000000000')) == '2.5e+1000000000000000000000'
    assert str(mpf('2.6e-1000000000000000000000')) == '2.6e-1000000000000000000000'
    assert str(mpf(1.23402834e-15)) == '1.23402834e-15'
    assert str(mpf(-1.23402834e-15)) == '-1.23402834e-15'
    assert str(mpf(-1.2344e-15)) == '-1.2344e-15'
    assert repr(mpf(-1.2344e-15)) == "mpf('-1.2343999999999999e-15')"

def test_pretty():
    mp.pretty = True
    assert repr(mpf(2.5)) == '2.5'
    assert repr(mpc(2.5,3.5)) == '(2.5 + 3.5j)'
    mp.pretty = False
    iv.pretty = True
    assert repr(mpi(2.5,3.5)) == '[2.5, 3.5]'
    iv.pretty = False

def test_str_whitespace():
    assert mpf('1.26 ') == 1.26

def test_unicode():
    mp.dps = 15
    try:
        unicode = unicode
    except NameError:
        unicode = str
    assert mpf(unicode('2.76')) == 2.76
    assert mpf(unicode('inf')) == inf

def test_str_format():
    assert to_str(from_float(0.1),15,strip_zeros=False) == '0.100000000000000'
    assert to_str(from_float(0.0),15,show_zero_exponent=True) == '0.0e+0'
    assert to_str(from_float(0.0),0,show_zero_exponent=True) == '.0e+0'
    assert to_str(from_float(0.0),0,show_zero_exponent=False) == '.0'
    assert to_str(from_float(0.0),1,show_zero_exponent=True) == '0.0e+0'
    assert to_str(from_float(0.0),1,show_zero_exponent=False) == '0.0'
    assert to_str(from_float(1.23),3,show_zero_exponent=True) == '1.23e+0'
    assert to_str(from_float(1.23456789000000e-2),15,strip_zeros=False,min_fixed=0,max_fixed=0) == '1.23456789000000e-2'
    assert to_str(from_float(1.23456789000000e+2),15,strip_zeros=False,min_fixed=0,max_fixed=0) == '1.23456789000000e+2'
    assert to_str(from_float(2.1287e14), 15, max_fixed=1000) == '212870000000000.0'
    assert to_str(from_float(2.1287e15), 15, max_fixed=1000) == '2128700000000000.0'
    assert to_str(from_float(2.1287e16), 15, max_fixed=1000) == '21287000000000000.0'
    assert to_str(from_float(2.1287e30), 15, max_fixed=1000) == '2128700000000000000000000000000.0'

def test_tight_string_conversion():
    mp.dps = 15
    # In an old version, '0.5' wasn't recognized as representing
    # an exact binary number and was erroneously rounded up or down
    assert from_str('0.5', 10, round_floor) == fhalf
    assert from_str('0.5', 10, round_ceiling) == fhalf

def test_eval_repr_invariant():
    """Test that eval(repr(x)) == x"""
    random.seed(123)
    for dps in [10, 15, 20, 50, 100]:
        mp.dps = dps
        for i in range(1000):
            a = mpf(random.random())**0.5 * 10**random.randint(-100, 100)
            assert eval(repr(a)) == a
    mp.dps = 15

def test_str_bugs():
    mp.dps = 15
    # Decimal rounding used to give the wrong exponent in some cases
    assert str(mpf('1e600')) == '1.0e+600'
    assert str(mpf('1e10000')) == '1.0e+10000'

def test_str_prec0():
    assert to_str(from_float(1.234), 0) == '.0e+0'
    assert to_str(from_float(1e-15), 0) == '.0e-15'
    assert to_str(from_float(1e+15), 0) == '.0e+15'
    assert to_str(from_float(-1e-15), 0) == '-.0e-15'
    assert to_str(from_float(-1e+15), 0) == '-.0e+15'

def test_convert_rational():
    mp.dps = 15
    assert from_rational(30, 5, 53, round_nearest) == (0, 3, 1, 2)
    assert from_rational(-7, 4, 53, round_nearest) == (1, 7, -2, 3)
    assert to_rational((0, 1, -1, 1)) == (1, 2)

def test_custom_class():
    class mympf:
        @property
        def _mpf_(self):
            return mpf(3.5)._mpf_
    class mympc:
        @property
        def _mpc_(self):
            return mpf(3.5)._mpf_, mpf(2.5)._mpf_
    assert mpf(2) + mympf() == 5.5
    assert mympf() + mpf(2) == 5.5
    assert mpf(mympf()) == 3.5
    assert mympc() + mpc(2) == mpc(5.5, 2.5)
    assert mpc(2) + mympc() == mpc(5.5, 2.5)
    assert mpc(mympc()) == (3.5+2.5j)

def test_conversion_methods():
    class SomethingRandom:
        pass
    class SomethingReal:
        def _mpmath_(self, prec, rounding):
            return mp.make_mpf(from_str('1.3', prec, rounding))
    class SomethingComplex:
        def _mpmath_(self, prec, rounding):
            return mp.make_mpc((from_str('1.3', prec, rounding), \
                from_str('1.7', prec, rounding)))
    x = mpf(3)
    z = mpc(3)
    a = SomethingRandom()
    y = SomethingReal()
    w = SomethingComplex()
    for d in [15, 45]:
        mp.dps = d
        assert (x+y).ae(mpf('4.3'))
        assert (y+x).ae(mpf('4.3'))
        assert (x+w).ae(mpc('4.3', '1.7'))
        assert (w+x).ae(mpc('4.3', '1.7'))
        assert (z+y).ae(mpc('4.3'))
        assert (y+z).ae(mpc('4.3'))
        assert (z+w).ae(mpc('4.3', '1.7'))
        assert (w+z).ae(mpc('4.3', '1.7'))
        x-y; y-x; x-w; w-x; z-y; y-z; z-w; w-z
        x*y; y*x; x*w; w*x; z*y; y*z; z*w; w*z
        x/y; y/x; x/w; w/x; z/y; y/z; z/w; w/z
        x**y; y**x; x**w; w**x; z**y; y**z; z**w; w**z
        x==y; y==x; x==w; w==x; z==y; y==z; z==w; w==z
    mp.dps = 15
    assert x.__add__(a) is NotImplemented
    assert x.__radd__(a) is NotImplemented
    assert x.__lt__(a) is NotImplemented
    assert x.__gt__(a) is NotImplemented
    assert x.__le__(a) is NotImplemented
    assert x.__ge__(a) is NotImplemented
    assert x.__eq__(a) is NotImplemented
    assert x.__ne__(a) is NotImplemented
    # implementation detail
    if hasattr(x, "__cmp__"):
        assert x.__cmp__(a) is NotImplemented
    assert x.__sub__(a) is NotImplemented
    assert x.__rsub__(a) is NotImplemented
    assert x.__mul__(a) is NotImplemented
    assert x.__rmul__(a) is NotImplemented
    assert x.__div__(a) is NotImplemented
    assert x.__rdiv__(a) is NotImplemented
    assert x.__mod__(a) is NotImplemented
    assert x.__rmod__(a) is NotImplemented
    assert x.__pow__(a) is NotImplemented
    assert x.__rpow__(a) is NotImplemented
    assert z.__add__(a) is NotImplemented
    assert z.__radd__(a) is NotImplemented
    assert z.__eq__(a) is NotImplemented
    assert z.__ne__(a) is NotImplemented
    assert z.__sub__(a) is NotImplemented
    assert z.__rsub__(a) is NotImplemented
    assert z.__mul__(a) is NotImplemented
    assert z.__rmul__(a) is NotImplemented
    assert z.__div__(a) is NotImplemented
    assert z.__rdiv__(a) is NotImplemented
    assert z.__pow__(a) is NotImplemented
    assert z.__rpow__(a) is NotImplemented

def test_mpmathify():
    assert mpmathify('1/2') == 0.5
    assert mpmathify('(1.0+1.0j)') == mpc(1, 1)
    assert mpmathify('(1.2e-10 - 3.4e5j)') == mpc('1.2e-10', '-3.4e5')
    assert mpmathify('1j') == mpc(1j)




try:
    import wheezy
    from wheezy_dynare_template import wheezy_dynare_template
    from wheezy_template import wheezy_template
except:
    pass

try:
    import jinja2
    from jinja2_template import jinja2_template
except:
    pass

try:
    import cheetah
    from cheetah_template import cheetah_template
except:
    pass
  
try:
    import mako
    from mako_dynare_template import mako_dynare_template
except:
    pass


from pymaclab.filters._bkfilter import bkfilter as bkf
import numpy as np

def bkfilter(data=None,up=6,dn=32,kkl=12):
    if type(data) == type(np.matlib.matrix([1,2,3])) and len(data.shape) == 2:
        if data.shape[0] < data.shape[1]: data = data.__array__().T
        else: data = data.__array__()
    elif type(data) != type(np.array([1,2,3])):
        data = np.array([x for x in data])
    elif type(data) == type(np.array([1,2,3])):
        if len(data.shape) == 2 and data.shape[0] < data.shape[1]:
            data = data.reshape(data.shape[1])
        elif len(data.shape) == 2 and data.shape[0] > data.shape[1]:
            data = data.reshape(data.shape[0])
    tlen = len(data)
    return bkf(data,up,dn,kkl,tlen)



version='0.2'
release=False

if not release:
    version += '.dev'
    import os
    svn_version_file = os.path.join(os.path.dirname(__file__),
                                    '__svn_version__.py')
    if os.path.isfile(svn_version_file):
        import imp
        svn = imp.load_module('sympycore.__svn_version__',
                              open(svn_version_file),
                              svn_version_file,
                              ('.py','U',1))
        version += svn.version




__all__ = ['POW']

from .base import BinaryHead, heads_precedence, Head, Expr, Pair, ArithmeticHead

from ..core import init_module
init_module.import_heads()
init_module.import_lowlevel_operations()
init_module.import_numbers()

@init_module
def _init(module):
    from ..arithmetic.numbers import try_power
    module.try_power = try_power

class PowHead(ArithmeticHead):
    """ PowHead represents exponentiation operation, data is a 2-tuple
    of base and exponent expressions. Both can be number instances or
    algebra instances.
    """
    op_mth = '__pow__'
    op_rmth = '__rpow__'

    def is_data_ok(self, cls, data):
        if type(data) is tuple and len(data)==2:
            base, exp = data
            if isinstance(base, cls):
                if isinstance(exp, numbertypes):
                    return
                if isinstance(exp, cls):
                    if exp.head is NUMBER:
                        if isinstance(exp.data, numbertypes):
                            return 'data[1] must be lowlevel number or non-numeric but got %s' % (type(exp.data))
                    else:
                        return
                else:
                    return 'data[1] must be %s instance but got %s' % ((cls, numbertypes), type(exp))
            else:
                return 'data[0] must be %s instance but got %s' % (cls, type(exp))
        else:
            return 'data must be 2-tuple'
        return

    def __repr__(self): return 'POW'

    def new(self, cls, (base, exp), evaluate=True):
        if exp==1:
            return base
        if exp==0 or base==1:
            return cls(NUMBER, 1)
        if not evaluate:
            return cls(self, (base, exp))
        if type(exp) is cls:
            h, d = exp.pair
            if h is NUMBER:
                exp = d
        if base.head is NUMBER and isinstance(exp, numbertypes):
            b = base.data
            if isinstance(b, numbertypes):
                r, base_exp_list = try_power(b, exp)
                if not base_exp_list:
                    return cls(NUMBER, r)
                if len(base_exp_list)==1:
                    b, e = base_exp_list[0]
                    rest = cls(POW, (cls(NUMBER, b), e))
                else:
                    d = {}
                    for b, e in base_exp_list:
                        d[cls(NUMBER, b)] = e
                    rest = cls(BASE_EXP_DICT, d)
                if r==1:
                    return rest
                return  cls(TERM_COEFF, (rest, r))
        return cls(self, (base, exp))

    def reevaluate(self, cls, (base, exp)):
        return base ** exp

    def data_to_str_and_precedence(self, cls, (base, exp)):
        pow_p = heads_precedence.POW
        div_p = heads_precedence.DIV
        if isinstance(base, Expr):
            b, b_p = base.head.data_to_str_and_precedence(cls, base.data)
        elif isinstance(base, numbertypes):
            b, b_p = NUMBER.data_to_str_and_precedence(cls, base)
        else:
            b, b_p = SYMBOL.data_to_str_and_precedence(cls, base)

        if isinstance(exp, Expr):
            h, d = exp.pair
            if h is NUMBER and isinstance(d, numbertypes):
                exp = d
        if isinstance(exp, numbertypes):
            if exp==0:
                return '1', heads_precedence.NUMBER
            if exp==1:
                return b, b_p
            if exp < 0:
                if exp==-1:
                    s1 = '('+b+')' if b_p <= pow_p else b
                    return '1/' + s1, div_p
                e, e_p = NUMBER.data_to_str_and_precedence(cls, -exp)
                s1 = '('+b+')' if b_p < pow_p else b
                s2 = '('+e+')' if e_p < pow_p else e
                return '1/' + s1 + '**' + s2, div_p
            e, e_p = NUMBER.data_to_str_and_precedence(cls, exp)
        else:
            if isinstance(exp, Expr):
                e, e_p = exp.head.data_to_str_and_precedence(cls, exp.data)
            else:
                e, e_p = str(exp), 0.0
        s1 = '('+b+')' if b_p <= pow_p else b
        s2 = '('+e+')' if e_p < pow_p else e
        return s1 + '**' + s2, pow_p

    def to_ADD(self, Algebra, (base, exp), expr):
        return Algebra(ADD, [expr])

    def to_TERM_COEFF_DICT(self, cls, data, expr):
        return expr

    def to_EXP_COEFF_DICT(self, cls, (base, exp), expr, variables=None):
        if isinstance(exp, Expr):
            if exp.head is NUMBER:
                exp = exp.data
            elif exp.head is TERM_COEFF:
                t. c = exp.data
                return self.to_EXP_COEFF_DICT(cls, (base**t, c), expr, variables)
        if isinstance(exp, inttypes):
            return base.head.to_EXP_COEFF_DICT(cls, base.data, base, variables) ** exp
        if isinstance(exp, rationaltypes):
            numer, denom = exp
            if numer!=1:
                return self.to_EXP_COEFF_DICT(cls, (base**(exp/numer), numer), expr, variables)
        raise NotImplementedError(`base, exp`)

    def non_commutative_mul(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return term_coeff_new(cls, (lhs, rdata))
        if rhead is SYMBOL or rhead is POW:
            return MUL.combine(cls, [lhs, rhs])
        if rhead is TERM_COEFF:
            term, coeff = rdata
            return (lhs * term) * coeff
        if rhead is DIFF:
            base, exp = lhs.data
            if base==rhs:
                return pow_new(cls, (base, exp + 1))
            return MUL.combine(cls, [lhs, rhs])
        if rhead is MUL:
            return MUL.combine(cls, [lhs] + rdata)
        raise NotImplementedError(`self, cls, lhs.pair, rhs.pair`)

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    non_commutative_mul_number = commutative_mul_number
    non_commutative_rmul_number = commutative_mul_number

    def commutative_mul(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return term_coeff_new(cls, (lhs, rdata))
        if rhead is SYMBOL or rhead is ADD or rhead is TERM_COEFF_DICT or rhead is APPLY or rhead is DIFF or rhead is FDIFF:
            lbase, lexp = lhs.data
            if lbase == rhs:
                return pow_new(cls, (lbase, lexp + 1))
            return cls(BASE_EXP_DICT, {rhs:1, lbase:lexp})
        if rhead is POW:
            lbase, lexp = lhs.data
            rbase, rexp = rdata
            if lbase==rbase:
                return POW.new(cls, (lbase, lexp + rexp))
            return cls(BASE_EXP_DICT, {lbase:lexp, rbase:rexp})
        if rhead is BASE_EXP_DICT:
            base, exp = lhs.data
            data = rhs.data.copy()
            dict_add_item(cls, data, base, exp)
            return base_exp_dict_new(cls, data)
        if rhead is TERM_COEFF:
            term, coeff = rdata
            return (lhs * term) * coeff
        raise NotImplementedError(`self, cls, lhs.pair, rhs.pair`)

    inplace_commutative_mul = commutative_mul

    def commutative_div_number(self, cls, lhs, rhs):
        r = number_div(cls, 1, rhs)
        if rhs==0:
            return r * lhs
        return term_coeff_new(cls, (lhs, r))

    def commutative_rdiv_number(self, cls, lhs, rhs):
        base, exp = lhs.data
        return term_coeff_new(cls, (pow_new(cls, (base, -exp)), rhs))

    def commutative_div(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return self.commutative_div_number(cls, lhs, rdata)
        base, exp = lhs.data
        if rhead is POW:
            rbase, rexp = rdata
            if base==rbase:
                return pow_new(cls, (base, exp-rexp))
            return base_exp_dict_new(cls, {base:exp, rbase: -rexp})
        if rhead is BASE_EXP_DICT:
            data = {base:exp}
            for b, e in rdata.iteritems():
                base_exp_dict_add_item(cls, data, b, -e)
            return base_exp_dict_new(cls, data)
        if rhead is TERM_COEFF:
            term, coeff = rhs.term_coeff()
            return (lhs / term) / coeff
        if base==rhs:
            return pow_new(cls, (base, exp-1))
        return base_exp_dict_new(cls, {base:exp, rhs:-1})
    
    def base_exp(self, cls, expr):
        base, exp = expr.data
        return base, exp

    def pow(self, cls, base, exp):
        if exp==0:
            return cls(NUMBER, 1)
        if exp==1:
            return base
        if isinstance(exp, Expr) and exp.head is NUMBER:
            exp = exp.data
        if isinstance(exp, inttypes):
            b, e = base.data
            base, exp = b, e*exp
        
        return POW.new(cls, (base, exp))

    pow_number = pow

    def walk(self, func, cls, data, target):
        base, exp = data
        base1 = base.head.walk(func, cls, base.data, base)
        if isinstance(exp, Expr):
            exp1 = exp.head.walk(func, cls, exp.data, exp)
        else:
            exp1 = NUMBER.walk(func, cls, exp, exp)
        if base1 is base and exp1 is exp:
            return func(cls, self, data, target)
        else:
            r = base1 ** exp1
            return func(cls, r.head, r.data, r)

    def scan(self, proc, cls, data, target):
        base, exp = data
        base.head.scan(proc, cls, base.data, target)
        if isinstance(exp, Expr):
            exp.head.scan(proc, cls, exp.data, target)
        else:
            NUMBER.scan(proc, cls, exp, target)
        proc(cls, self, data, target)

    def expand(self, cls, expr):
        base, exp = expr.data
        if isinstance(exp, Expr):
            exp = exp.expand()
            h, d = exp.pair
            if h is NUMBER and isinstance(d, int):
                exp = d
        if isinstance(base, Expr):
            base = base.expand()
            if isinstance(exp, int):
                return base.head.expand_intpow(cls, base, exp)
        return cls(POW, (base, exp))

    def diff(self, cls, data, expr, symbol, order, cache={}):
        # XXXX needs implementatiin
        key = (expr, symbol, order)
        result = cache.get(key)
        if result is not None:
            return result
        base, exp = data
        texp = type(exp)
        if symbol not in base.symbols_data:
            # constant ** exp
            if texp is cls:
                if exp.head is SYMBOL:
                    if exp.data==symbol:
                        result = expr * cls.Log(base)**order
                        cache[key] = result
                        return result
                    else:
                        return cls(NUMBER, 0)
                if symbol not in exp.symbols_data:
                    return cls(NUMBER, 0)
                key1 = (expr, symbol, 1)
                result = cache.get(key1)
                if result is None:
                    de = exp.head.diff(cls, exp.data, exp, symbol, 1, cache=cache)
                    if symbol not in de.symbols_data:
                        result = expr * de**order * cls.Log(base)**order
                        cache[key] = result
                        return result
                    result = expr * cls.Log(base) * de
                    cache[key1] = result
                if order>1:
                    result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
                    cache[key] = result
                return result
            else:
                return cls(NUMBER, 0)
        elif not (texp is cls and symbol in exp.symbols_data):
            if exp is cls and exp.head is NUMBER:
                exp = exp.data
            # variable ** constant            
            # f(x)**n  -> n*f**(n-1)*f' ->
            db = base.head.diff(cls, base.data, base, symbol, 1, cache=cache)
            if db.head is NUMBER:
                if texp is int and order>exp and exp>0:
                    return cls(NUMBER, 0)            
                p = db.data ** order
                e = exp
                for i in xrange(order):
                    p *= e
                    e -= 1
                result = p * base ** e
                cache[key] = result
                return result

        key1 = (expr, symbol, 1)
        result = cache.get(key1)
        if result is None:
            base, exp = data
            db = base.head.diff(cls, base.data, base, symbol, 1, cache=cache)            
            if isinstance(exp, Expr):
                de = exp.head.diff(cls, exp.data, exp, symbol, 1, cache=cache)
                if de==0:
                    result = base ** (exp-1) * db * exp
                else:
                    result = expr * (db / base * exp + cls.Log(base) * de)
            else:
                result = base ** (exp-1) * db * exp
            cache[key1] = result
        if order>1:
            # todo: if, say, order>400, return unevaluated result to avoid recursion
            result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
            cache[key] = result
        return result

    def diff_apply(self, cls, data, diff, expr):
        base, exp = data
        bhead, bdata = base.pair
        if bhead is DIFF and isinstance(exp, inttypes) and exp>=0:
            return expr.head.diff(type(expr), expr.data, expr, bdata.data, exp)
        if bhead is FDIFF:# and isinstance(exp, inttypes) and exp>=0:
            return expr.head.fdiff(type(expr), expr.data, expr, bdata.data, exp)
        return NotImplemented

    def apply(self, cls, data, func, args):
        base, exp = data
        if isinstance(exp, Expr):
            return NotImplemented
        return base.head.apply(cls, base.data, base, args) ** exp

    def integrate_indefinite(self, cls, data, expr, x):
        base, exp = data
        t = type(exp)
        if t is cls:
            ehead, edata = exp.pair
            exp_symbols_data = exp.symbols_data
        else:
            ehead, edata = NUMBER, exp
            exp_symbols_data = set([])
        bhead, bdata = base.pair
        if bhead is SYMBOL and bdata==x and x not in exp_symbols_data:
            e1 = edata + 1
            return pow_new(cls, (base, e1))/e1
        if bhead is NUMBER or x not in base.symbols_data:
            if ehead is SYMBOL and edata == x:
                return expr / cls.Log(base)
            elif ehead is TERM_COEFF:
                t, c = edata
                if t.head is SYMBOL and t.data==x:
                    return expr / (cls.Log(base) * c)
            if x not in exp_symbols_data:
                return expr * cls(SYMBOL, x)
        raise NotImplementedError("don't know how to integrate %s over %s" % (expr, x))

    def integrate_definite(self, cls, data, expr, x, a, b):
        base, exp = data
        t = type(exp)
        if t is cls:
            ehead, edata = exp.pair
            exp_symbols_data = exp.symbols_data
        else:
            ehead, edata = NUMBER, exp
            exp_symbols_data = set([])
        bhead, bdata = base.pair
        if bhead is SYMBOL and bdata==x and x not in exp_symbols_data:
            e1 = edata + 1
            return (b**e1 - a**e1)/e1
        if bhead is NUMBER or x not in base.symbols_data:
            if ehead is SYMBOL and edata == x:
                return (base**b - base**a) / cls.Log(base)
            elif ehead is TERM_COEFF:
                t, c = edata
                if t.head is SYMBOL and t.data==x:
                    return (base**(b*c) - base**(a*c)) / (cls.Log(base) * c)
            if x not in exp_symbols_data:
                return expr * (b - a)
        raise NotImplementedError("don't know how to integrate %s over %s in [%s, %s]" % (expr, x, a, b))

    def algebra_mul_number(self, Algebra, lhs, rhs, inplace):
        return self.algebra_mul(Algebra, lhs, Algebra(NUMBER, rhs), inplace)
    
    def algebra_mul(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('is_multiplicative_group'):
            if Algebra.algebra_options.get('evaluate_multiplication'):
                rhead, rdata = rhs.pair
                if Algebra.algebra_options.get('is_multiplicative_group_commutative'):
                    pass
                else:
                    if rhead is MUL:
                        data = [lhs] + rdata
                    else:
                        data = [lhs, rhs]
                    coeff = MUL.combine_mul_list(Algebra, data)
                    assert coeff is 1,`coeff`
                    return mul_new(Algebra, data)
                        
        return super(type(self), self).algebra_mul(Algebra, lhs, rhs, inplace)

    def algebra_pow_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('evaluate_multiplication'):
            if rhs==1:
                return lhs
            if not rhs:
                return Algebra(NUMBER, 1)
            if rhs==-1:
                base, exp = lhs.data
                return pow_new(Algebra, (base, -exp))
        return super(type(self), self).algebra_pow_number(Algebra, lhs, rhs, inplace)
POW = PowHead()



from sympycore.calculus import *

x, y = map(Symbol, 'xy')

def test_integrate():
    assert Number(3).integrate(x) == 3*x
    assert x.integrate(x) == x**2 / 2
    assert (2*x).integrate(x) == x**2
    assert (x*y).integrate(x) == y * x**2 / 2
    assert (x*y**2).integrate(x) == y**2 * x**2 / 2
    r1 = (1 + 4*y*x + 3*x**2).integrate(x)
    r2 = x + 2*y*x**2 + x**3
    assert r1 == r2
    failures = [x**x, x**(1+y+5**x), 1/(2+x), x/(2+3*x+5), (3*x+2*y)*x][:-1]
    for f in failures:
        try:
            r = f.integrate(x)
            assert 0, ("integration of %s expected to fail but got %s" % (f, r))
        except NotImplementedError:
            pass

def test_integrate_defined():
    for g in [Number(4), 2*x, 6*x**2, 2*x+100*x**3]:
        f = g.integrate(x)
        f1 = f.subs(x,5)-f.subs(x,2)
        f2 = g.integrate((x,2,5))
        assert f2==f1,`str(g), str(f), str(f1), str(f2)`



# -*- coding: latin-1 -*-
"""
  This module implements extension type Expr that holds two Python
  objects, head and data, in a pair attribute.

  When adding new features to Expr class, make sure that these are
  added also to extension type Expr in src/expr_ext.c.

C Expr:

>>> from sympycore.expr_ext import *
>>> %timeit Expr(1,2)
10000000 loops, best of 3: 179 ns per loop
>>> e=Expr(1,2)
>>> %timeit h = e.head
10000000 loops, best of 3: 113 ns per loop
>>> %timeit h,d = e.pair
10000000 loops, best of 3: 141 ns per loop

Python Expr:

>>> from sympycore.expr import *
>>> %timeit Expr(1,2)
1000000 loops, best of 3: 988 ns per loop
>>> e=Expr(1,2)
>>> %timeit h = e.head
10000000 loops, best of 3: 119 ns per loop
>>> %timeit h,d = e.pair
10000000 loops, best of 3: 139 ns per loop
"""
# Author: Pearu Peterson
# Created: March 2008

__all__ = ['Expr', 'Pair']

def init_module(m):
    from .core import heads
    for n,h in heads.iterNameValue(): setattr(m, n, h)
    from .arithmetic.numbers import inttypes, numbertypes, try_power, gcd
    m.inttypes = inttypes
    m.numbertypes = numbertypes
    m.try_power = try_power
    m.gcd = gcd

class Expr(object):
    """Represents an symbolic expression in a pair form: (head, data)	
                									
The pair (head, data) is saved in an attribute ``pair``. The parts of
a pair, head and data, can be also accessed via ``head`` and ``data``
attributes, respectively. All three attributes are read-only.
  									
The head part is assumed to be an immutable object.  The data part can
be either an immutable object or Python dictionary.  In the former
case, the hash value of Expr instance is defined as::
  									
    hash((<Expr>.head, <Expr>.

Otherwise, if ``data`` contains a Python dictionary, then the hash
value is defined as::
  									
    hash((<Expr>.head, frozenset(<Expr>.data.items())

If ``data`` is a Python list, then the hash value is::

  hash((<Expr>.head, tuple(<Expr>.data)))

WARNING: the hash value of an Expr instance is computed (and cached)
when it is used as a key to Python dictionary. This means that the
instance content MUST NOT be changed after the hash is computed.  To
check if it is safe to change the ``data`` dictionary, use
``is_writable`` attribute that returns True when the hash value has
not been computed::
  									
    <Expr>.is_writable -> True or False				
  									
There are two ways to access the parts of a Expr instance from	
Python::								
  									
    a = Expr(<head>, <data>)					
    head, data = a.head, a.data     - for backward compatibility	
    head, data = a.pair             - fastest way			

When Expr constructor is called with one argument, say ``x``, then
``<Expr subclass>.convert(x)`` will be returned.

This is Python version of Expr type.
"""

    __slots__ = ['head', 'data', 'pair', '_hash']

    def __init__(self, *args):
        if len(args)==1:
            obj = self.convert(args[0])
            self.pair = obj.pair
            self._hash = obj._hash
        elif len(args)==2:
            self.pair = args
            self._hash = None
        else:
            raise TypeError("%s requires 1 or 2 arguments but got %r" % (type(self), len(args)))
        msg = self.head.is_data_ok(type(self), self.data)
        if msg:
            msg = '%s(head=%s, data=%s): %s' % (type(self).__name__, self.head, self.data, msg)
            #print msg
            raise TypeError(msg)
        
    def __repr__(self):
        return '%s%r' % (type(self).__name__, self.pair)

    def __hash__(self):
        """ Compute hash value.
        
        Different from expr_ext.Expr, an exception is raised when data
        dictionary values contain dictionaries.
        """
        h = self._hash
        if h is None:
            pair = self.pair
            obj = self.as_lowlevel()
            if obj is not pair:
                h = hash(obj)
            else:
                head, data = pair
                tdata = type(data)
                if tdata is dict:
                    h = hash((head, frozenset(data.iteritems())))
                elif tdata is list:
                    h = hash((head, tuple(data)))
                else:
                    h = hash(pair)
            self._hash = h
        return h

    @property
    def is_writable(self, _writable_types = (list, dict)):
        if self._hash is None:
            data = self.pair[1]
            tdata = type(data)
            if tdata in _writable_types:
                return True
            if tdata is Pair:
                return data.is_writable
        return False

    @property
    def head(self):
        return self.pair[0]

    @property
    def data(self):
        return self.pair[1]

    # Pickle support:
    def _sethash(self, hashvalue):
        """ Set hash value for the object.

        If hashvalue==-1, then the hash value will be reset.

        Used by pickle support in sympycore.core._reconstruct. DO NOT
        use this method directly.
        """
        if hashvalue==-1:
            self._hash = None
        else:
            self._hash = hashvalue

    def __reduce__(self):
        # see also _reconstruct function in sympycore/core.py
        version = 3
        from sympycore.core import _reconstruct
        if version==1:
            hashvalue = self._hash
            if hashvalue is None:
                hashvalue = -1
            state = (type(self), self.pair, hashvalue)
        elif version==2 or version==3:
            hashvalue = self._hash
            if hashvalue is None:
                hashvalue = -1
            cls = type(self)
            typ = type(cls)
            try:
                args = typ.__getinitargs__(cls)
            except AttributeError:
                args = None
            if args is None:
                # either metaclass does not define __getinitargs__ method
                # or cls has no metaclass
                state = (cls, self.pair, hashvalue)
            else:
                state = ((typ, args), self.pair, hashvalue)
        else:
            raise NotImplementedError('pickle state version %s' % (version))
        return  _reconstruct, (version, state)

    def __nonzero__(self):
        # Note that `not cls(MUL, [])` would return True while `cls(MUL, [])==1`.
        # So, must use as_lowlevel:
        obj = self.as_lowlevel()
        if obj is not self.pair:
            return not not obj
        return not not self.data

    def as_lowlevel(self):
        """ Return self as low-level object instance that will be used
        in comparison and in hash computation.

        By default, as_lowlevel uses heads to_lowlevel(cls, data, pair)
        method but since as_lowlevel is a most frequently called
        method then for some heads the corresponding code is copied
        here. The default return value is a pair tuple for composite
        objects and data part for atomic objects. The as_lowlevel
        method may also return an Expr instance but not self
        (otherwise infinite recurrsion will occur).

        See __hash__, __nonzero__ method for more details how the
        results of as_lowlevel method are interpreted.
        """
        head, data = pair = self.pair
        if head is NUMBER or head is SYMBOL or head is SPECIAL:
            return data
        elif head is MUL or head is DIV:
            n = len(data)
            if n==0:
                return 1
            if n==1:
                return data[0]
        elif head is ADD or head is SUB:
            n = len(data)
            if n==0:
                return 0
            if n==1:
                return data[0]
        elif head is POW:
            base, exp = data
            if exp==0 or base==1:
                return 1
            if exp==1:
                return base
        elif head is TERM_COEFF:
            term, coeff = data
            if coeff==0:
                return 0
            if coeff==1:
                return term
            if term==1:
                return coeff
        elif head is TERM_COEFF_DICT:
            n = len(data)
            if n==0:
                return 0
            if n==1:
                return type(self)(TERM_COEFF, dict_get_item(data))
        elif head is BASE_EXP_DICT:
            n = len(data)
            if n==0:
                return 1
            if n==1:
                return type(self)(POW, dict_get_item(data))
        else:
            return head.to_lowlevel(type(self), data, pair)
        return pair

    def term_coeff(self):
        head, data = self.pair
        if head is TERM_COEFF:
            return data
        if head is BASE_EXP_DICT:
            cls = type(self)
            coeff = base_exp_dict_get_coefficient(cls, data)
            if coeff is not None:
                d = data.copy()
                del d[coeff]
                r = BASE_EXP_DICT.new(cls, d)
                return r, coeff.data
                t, c = r.head.term_coeff(cls, r)
                return t, c * coeff 
            return self, 1
        if head is NUMBER:
            return self, data
        return self, 1

    def base_exp(self):
        head, data = self.pair
        if head==POW:
            return data
        return self, 1

    for _item in dict(__eq__ = '==', __ne__ = '!=',
                      __lt__ = '<', __le__ = '<=',
                      __gt__ = '>', __ge__ = '>=',
                      ).items():
        exec '''
def %s(self, other):
    if type(self) is type(other):
        other = other.as_lowlevel()
    return self.as_lowlevel() %s other
''' % _item


class Pair(Expr):
    """ Holds a pair that may contain list and dict second element.
    """
    def __init__(self, *args):
        if len(args)==2:
            self.pair = args
            self._hash = None
        else:
            raise TypeError("%s requires 2 arguments but got %r" % (type(self), len(args)))

    def __eq__(self, other):
        return self.pair == other

    def __len__(self):
        return 2
    
    def __getitem__(self, index):
        return self.pair[index]


def dict_mul_item(Algebra, d, key, value):
    c = d.get(key)
    if c is None:
        d[key] = value
    else:
        d[key] = c * value

base_exp_dict_mul_item = dict_mul_item

def term_coeff_dict_mul_item(Algebra, d, key, value):
    return dict_mul_item(Algebra, d, key, value)

def dict_add_item(Algebra, d, key, value):
    c = d.get(key)
    if c is None:
        if value:
            d[key] = value
    else:
        c = c + value
        if c:
            d[key] = c
        else:
            del d[key]

def base_exp_dict_get_coefficient(Algebra, d):
    for k, v in d.iteritems():
        if v is 1 and k.head is NUMBER:
            return k
    return

def base_exp_dict_add_item(Algebra, d, base, exp):
    """
    d is a dictonary that will be updated with base, exp pair.
    Base part must be an Algebra instance while exp part
    can either be a number instance or an expression.

    The dictionary items (base, exp) must satisfy the following
    conditions:
      1) numeric (base, exp) must be completely evaluated, that is,
      if base is numeric then exp is either 1 or rational with denominator
      part that is not a root of base.
      2) all numeric items with exp==1 must be combined to one item
      (coefficient)
      3) items with base==1 and exp==0 must not be present in the dictionary
    """
    base_head, base_data = base.pair
    value = d.get(base)
    if type(exp) is Algebra and exp.head is NUMBER:
        exp = exp.data
    if value is None:
        if base_head is NUMBER:
            assert base
            if base==1:
                return
            if exp==1:
                coeff = base_exp_dict_get_coefficient(Algebra, d)
                if coeff is None:
                    d[base] = 1
                else:
                    del d[coeff]
                    coeff = coeff * base
                    if coeff==1:
                        return
                    d[coeff] = 1
            else:
                assert not isinstance(exp, inttypes),`base, exp`
                d[base] = exp
        else:
            assert exp
            d[base] = exp
    else:
        value = value + exp
        if type(value) is Algebra and value.head is NUMBER:
            value = value.data
        if value:
            if base_head is NUMBER and isinstance(value, numbertypes):
                del d[base]
                r, l = try_power(base_data, value)
                if r!=1:
                    base_exp_dict_add_item(Algebra, d, Algebra(NUMBER, r), 1)
                elif len(l)==1 and l[0]==base_data:
                    d[base] = value
                else:
                    for b, e in l:
                        base_exp_dict_add_item(Algebra, d, Algebra(NUMBER, b), e)
            elif base_head is TERM_COEFF and isinstance(value, inttypes):
                del d[base]
                term, coeff = base_data
                base_exp_dict_add_item(Algebra, d, term, value)
                base_exp_dict_add_item(Algebra, d, Algebra(NUMBER, coeff), value)
            else:
                d[base] = value
        else:
            del d[base]

def term_coeff_dict_add_item(Algebra, d, key, value):
    khead, kdata = key.pair
    if khead is TERM_COEFF:
        term, coeff = kdata
        dict_add_item(Algebra, d, term, value * coeff)
    else:
        assert khead not in [TERM_COEFF_DICT],`Algebra, key.pair`
        dict_add_item(Algebra, d, key, value)

def term_coeff_dict_add_dict(Algebra, dict1, dict2):
    for key, value in dict2.iteritems():
        term_coeff_dict_add_item(Algebra, dict1, key, value)

def dict_get_item(d):
    return d.items()[0]

def dict_add_dict(Algebra, dict1, dict2):
    for key, value in dict2.iteritems():
        dict_add_item(Algebra, dict1, key, value)

def base_exp_dict_add_dict(Algebra, dict1, dict2):
    for key, value in dict2.iteritems():
        base_exp_dict_add_item(Algebra, dict1, key, value)

def base_exp_dict_sub_item(Algebra, dict, key, value):
    return base_exp_dict_add_item(Algebra, dict, key, -value)

def base_exp_dict_sub_dict(Algebra, dict1, dict2):
    for key, value in dict2.iteritems():
        base_exp_dict_sub_item(Algebra, dict1, key, value)

def dict_sub_dict(Algebra, dict1, dict2):
    for key, value in dict2.iteritems():
        dict_add_item(Algebra, dict1, key, -value)

def base_exp_dict_mul_dict(Algebra, d, dict1, dict2):
    for t1,c1 in dict1.iteritems():
        for t2,c2 in dict2.iteritems():
            t = t1 * t2
            t, c = t.term_coeff()
            c12 = c * c1 * c2
            base_exp_dict_add_item(Algebra, d, t, c12)

def exp_coeff_dict_mul_dict(Algebra, d, dict1, dict2):
    for t1,c1 in dict1.iteritems():
        for t2,c2 in dict2.iteritems():
            t = t1 + t2
            c12 = c1 * c2
            dict_add_item(Algebra, d, t, c12)
            
def dict_mul_dict(Algebra, d, dict1, dict2):
    for t1,c1 in dict1.iteritems():
        for t2,c2 in dict2.iteritems():
            t = t1 * t2
            #t, c = t.term_coeff()
            c12 = c1 * c2
            dict_add_item(Algebra, d, t, c12)
    
def dict_mul_value(Algebra, d, value):
    if value==1:
        return
    for t, c in d.items():
        c *= value
        if c:
            d[t] = c
        else:
            del d[t]

term_coeff_dict_mul_dict = dict_mul_dict
base_exp_dict_mul_value = dict_mul_value
term_coeff_dict_mul_value = dict_mul_value

def term_coeff_new(Algebra, data):
    term, coeff = data
    if coeff==1:
        return term
    if term==1:
        return Algebra(NUMBER, coeff)
    if coeff==0:
        return Algebra(NUMBER, 0)
    h,d = term.pair
    if h is TERM_COEFF:
        t, c = d
        return term_coeff_new(Algebra, (t, c * coeff))
    if h is NUMBER:
        return Algebra(NUMBER, d * coeff)
    return Algebra(TERM_COEFF, data)

def term_coeff_dict_new(Algebra, data):
    """
    Return canonicalized TERM_COEFF_DICT expression from data.
    """
    n = len(data)
    if n>1:
        return Algebra(TERM_COEFF_DICT, data)
    if n==0:
        return Algebra(NUMBER, 0)
    return term_coeff_new(Algebra, dict_get_item(data))

def term_coeff_dict(Algebra, expr):
    """
    Return canonicalized TERM_COEFF_DICT expression from existing
    expression:
    
      * if expr has no data, return 0
      * if expr data has one item, return TERM_COEFF expression
    """
    data = expr.data
    n = len(data)
    if n>1:
        return expr
    if n==0:
        return Algebra(NUMBER, 0)
    return term_coeff_new(Algebra, dict_get_item(data))

def pow_new(Algebra, data):
    base, exp = data
    if exp==1:
        return base
    if base==1 or exp==0:
        return Algebra(NUMBER, 1)
    #if type(exp) is Algebra and exp.head is NUMBER:
    #    data = base, exp.data
    return Algebra(POW, data)

def base_exp_dict_new(Algebra, data):
    n = len(data)
    if n==0:
        return Algebra(NUMBER, 1)
    if n==1:
        return pow_new(Algebra, dict_get_item(data))
    coeff = base_exp_dict_get_coefficient(Algebra, data)
    if coeff is None:
        return Algebra(BASE_EXP_DICT, data)
    del data[coeff]
    return term_coeff_new(Algebra, (base_exp_dict_new(Algebra, data), coeff.data))

def add_new(Algebra, data):
    n = len(data)
    if n==0:
        return Algebra(NUMBER, 0)
    if n==1:
        return data[0]
    return Algebra(ADD, data)

def add(Algebra, expr):
    data = expr.data
    n = len(data)
    if n==0:
        return Algebra(NUMBER, 0)
    if n==1:
        return data[0]
    return expr

def mul_new(Algebra, data):
    n = len(data)
    if n==0:
        return Algebra(NUMBER, 1)
    if n==1:
        return data[0]
    return Algebra(MUL, data)



"""Low-level arithmetics support.

Arithmetic package provides:

  * low-level number types ``mpq``, ``mpf``, ``mpqc``, ``mpc``
  * base class ``Infinity`` for extended numbers
  * various low-level number theory functions: ``gcd``, ``multinomial_coefficients``, etc.

"""
__docformat__ = "restructuredtext"

from . import mpmath
from .numbers import mpq, mpf, mpqc, mpc, setdps, getdps
from .number_theory import gcd, lcm, multinomial_coefficients, f2q
from .infinity import Infinity



from pymaclab.filters._cffilter import cffilter as cff
import numpy as np

def cffilter(data=None,low=6,high=32,drift=True):
    if type(data) == type(np.matlib.matrix([1,2,3])) and len(data.shape) == 2:
        if data.shape[0] < data.shape[1]: data = data.__array__().T
        else: data = data.__array__()
    elif type(data) != type(np.array([1,2,3])):
        data = np.array([x for x in data])
    elif type(data) == type(np.array([1,2,3])):
        if len(data.shape) == 2 and data.shape[0] < data.shape[1]:
            data = data.reshape(data.shape[1])
        elif len(data.shape) == 2 and data.shape[0] > data.shape[1]:
            data = data.reshape(data.shape[0])
    return cff(data,low=low,high=high,drift=drift) 



import numpy
import copy
from ..linalg import pca_module

def genXX(self,matd=None,nlags=None,const='standard',func=False):
    # Get all the needed variables from instance
    if nlags == None: nlags = copy.deepcopy(self.nlags)
    if matd == None: matd = copy.deepcopy(self.data)
    if const == 'standard': const = copy.deepcopy(self.confdic['const_term'])
    rows = matd.shape[0]
    cols = matd.shape[1]
    nmatd = numpy.zeros([rows-nlags,cols*nlags])
    for lag in range(0,nlags,1):
        for col in range(0,cols,1):
            shift = lag*(cols-1)
            if lag == 0:
                nmatd[:,col+shift+lag] = matd[lag:-(nlags-(lag)),col]
            elif lag == nlags - 1:
                nmatd[:,col+shift+lag] = matd[lag:-1,col]
            elif lag > 0:
                nmatd[:,col+shift+lag] = matd[lag:-(nlags-(lag)),col]
    # Flip matrix left to right in order to have p(1) lags first, then re-order variables
    nmatd = numpy.fliplr(nmatd)
    for elem in range(0,nlags,1):
        nmatd[:,elem*cols:(elem+1)*cols] = numpy.fliplr(nmatd[:,elem*cols:(elem+1)*cols])
    # Add the colum for the constants
    if const == 'tt':
        constar = numpy.reshape(numpy.array([x for x in range(0,nmatd.shape[0],1)]),(nmatd.shape[0],1))
        nmatd = numpy.hstack((constar,nmatd))
    elif const == 'cc':
        constar = numpy.reshape(numpy.array([1,]*nmatd.shape[0]),(nmatd.shape[0],1))
        nmatd = numpy.hstack((constar,nmatd))
    elif const == 'ct':
        constar = numpy.reshape(numpy.array([1,]*nmatd.shape[0]),(nmatd.shape[0],1))
        consttrend = numpy.reshape(numpy.array([x for x in range(0,nmatd.shape[0],1)]),(nmatd.shape[0],1))
        nmatd = numpy.hstack((constar,consttrend,nmatd))
    elif const == 'ctt':
        constar = numpy.reshape(numpy.array([1,]*nmatd.shape[0]),(nmatd.shape[0],1))
        consttrend = numpy.reshape(numpy.array([x for x in range(0,nmatd.shape[0],1)]),(nmatd.shape[0],1))
        consttrend2 = consttrend**2
        nmatd = numpy.hstack((constar,consttrend,consttrend2,nmatd))
    if not func:
        self.nmatd = nmatd
        return self
    elif func:
        return nmatd
    

def is_stable(compmat=None,verbose=False):
    """
Determine stability of VAR(p) system by examining the eigenvalues of the
VAR(1) representation

Parameters
----------
compmat : ndarray (p x k,p x k)

Returns
-------
is_stable : bool
"""
    eigs = numpy.linalg.eigvals(compmat)

    if verbose:
        print 'Eigenvalues of VAR(1) rep'
        for val in numpy.abs(eigs):
            print val

    return (numpy.abs(eigs) <= 1).all()


def genPCs(self,data=None,sfacs=None,func=False):
    '''
    Using PCA to extract the factors from some array of data
    This utility function uses non-stacked vectors or data, s stands for simple
    '''
    if sfacs == None: npc = copy.deepcopy(self.sfacs)
    else: npc = sfacs
    cols = self.ncols
    nlags = self.nlags
    pcm = pca_module.PCA_svd(data,standardize=False)[0][:,:npc]
    exp_var = pca_module.PCA_svd(data,standardize=False)[2][:npc]
    pcmm = numpy.zeros((len(pcm),len(pcm[0])))
    for factor in range(0,len(pcm),1):
        pcmm[factor,:] = pcm[factor]
    if not func:
        self.fdata = copy.deepcopy(pcmm)
        self.exp_var = copy.deepcopy(exp_var)
    elif func:
        return pcmm,exp_var



def genPC(self,data=None,func=False):
    '''
    Using PCA to extract the factors from some array of data
    This utility function uses the stacked data instead
    '''
    npc = copy.deepcopy(self.sfacs)
    cols = self.ncols
    flags = self.flags
    nlags = self.nlags
    stackX = copy.deepcopy(data)
    pcm = numpy.zeros((stackX.shape[0],npc*flags))
    for lago in range(0,flags,1):
        pcm[:,lago*npc:(lago+1)*npc] = pca_module.PCA_svd(stackX[:,lago*cols:(lago+1)*cols],standardize=False)[0][:,:npc]
    if not func:
        self.fdata = copy.deepcopy(pcm)
    elif func:
        return pcm

def compbetta(self,matd=None,nmatd=None,smbetta=True,const='standard',func=False):
    # Get all the needed variables from instance
    if matd == None: matd = self.data
    if nmatd == None: nmatd = self.nmatd
    cols = matd.shape[1]
    if const == 'standard': const = self.confdic['const_term']
    if const == None:
        nlags = int(nmatd.shape[1]/cols)
    elif const == 'cc':
        nlags = int((nmatd.shape[1]-1)/cols)
    elif const == 'tt':
        nlags = int((nmatd.shape[1]-1)/cols)
    elif const == 'ct':
        nlags = int((nmatd.shape[1]-2)/cols)
    elif const == 'ctt':
        nlags = int((nmatd.shape[1]-3)/cols)
    XX = numpy.dot(nmatd.T,nmatd)
    XXi = numpy.linalg.inv(XX)
    if const == None:
        betta = numpy.zeros((cols,nlags*cols))
    elif const == 'tt' or const == 'cc':
        betta = numpy.zeros((cols,1+nlags*cols))
    elif const == 'ct':
        betta = numpy.zeros((cols,2+nlags*cols))
    elif const == 'ctt':
        betta = numpy.zeros((cols,3+nlags*cols))
    for vari in range(0,cols,1):
        ymat=matd[nlags:,vari]
        Xy = numpy.dot(nmatd.T,ymat)
        betta[vari,:] = numpy.dot(XXi,Xy)
    if smbetta:
        # Produce the betta format from statsmodels library so we can use some of their code
        # Exclude the trend or constant from bettasm, as opposed to in betta
        bettasm = numpy.zeros((nlags,cols,cols))
        for lago in range(0,nlags,1):
            for varos in range(0,cols,1):
                if const == 'tt' or const == 'cc':
                    bettasm[lago][varos,:] = betta[:,1:][varos][cols*lago:cols*(lago+1)]
                elif const == None:
                    bettasm[lago][varos,:] = betta[varos][cols*lago:cols*(lago+1)]
                elif const == 'ct':
                    bettasm[lago][varos,:] = betta[:,2:][varos][cols*lago:cols*(lago+1)]
                elif const == 'ctt':
                    bettasm[lago][varos,:] = betta[:,3:][varos][cols*lago:cols*(lago+1)]
    if not func:
        if smbetta:
            self.bettasm = bettasm
            self.betta = betta
            return self
        elif not smbetta:
            self.betta = betta
            return self
    elif func:
        if smbetta:
            return bettasm
        elif not smbetta:
            return betta

def check_matrix(arraym=None,arrname=None,expdim=None):
    if type(arraym) == type(numpy.array([[1,2,3],[1,2,3]])):
        rows = arraym.shape[0]
        cols = arraym.shape[1]
    elif type(arraym) == type({}):
        cols = len(arraym[arraym.keys()[0]])
        rows = len(arraym.keys())
    print '%%%%%%%%%%%%%%% MATRIX DIM CHECK %%%%%%%%%%%%%%%%%%'
    print 'Matrix ##'+arrname+'## has dim ('+str(rows)+','+str(cols)+')'
    if expdim != None:
        print 'The expected dimension was: ('+str(expdim[0])+','+str(expdim[1])+')'
        if (rows,cols) == expdim: print 'Matrix dim check PASSED !'
    print '%%%%%%%%%%%%%%% DIM CHECK END %%%%%%%%%%%%%%%%%%%%%%'

  
def genStackX(self,data=None,flags=None,func=False):
    '''
    Utility function used to take data X (which may be n-dimensional on horizontal axis)
    and then to create a stacked vector based on flags 
    '''
    # Get all the needed variables from instance
    if flags == None:
        flags = copy.deepcopy(self.flags)
    matd = copy.deepcopy(data)
    rows = matd.shape[0]
    cols = matd.shape[1]
    vnames = copy.deepcopy(self.vnames)
    nmatd = numpy.zeros([rows-flags,cols*flags])
    for lag in range(0,flags,1):
        for col in range(0,cols,1):
            shift = lag*(cols-1)
            if lag == 0:
                nmatd[:,col+shift+lag] = matd[lag:-(flags-lag),col]
            elif lag == flags - 1:
                nmatd[:,col+shift+lag] = matd[lag:-1,col]
            elif lag > 0:
                nmatd[:,col+shift+lag] = matd[lag:-(flags-lag),col]
    # Flip matrix left to right in order to have p(1) lags first, then re-order variables
    nmatd = numpy.fliplr(nmatd)
    for elem in range(0,flags,1):
        nmatd[:,elem*cols:(elem+1)*cols] = numpy.fliplr(nmatd[:,elem*cols:(elem+1)*cols])
    # Add original data back to lags in first columns to get stacked X with current and lagged
    nmatd = numpy.hstack((matd[flags:,:],nmatd))
    if not func:
        self.stackmat = copy.deepcopy(nmatd)
        return self
    elif func:
        return nmatd


  
def genStackX2(self,data=None,flags=None,fleads=None,func=False):
    '''
    Utility function used to take data X (which may be n-dimensional on horizontal axis)
    and then to create a stacked vector based on flags WORK IN PROGRESS 
    '''
    # Get all the needed variables from instance
    if flags == None:
        flags = copy.deepcopy(self.flags)
    if fleads == None:
        fleads = 0
    matd = copy.deepcopy(data)
    rows = matd.shape[0]
    cols = matd.shape[1]
    vnames = copy.deepcopy(self.vnames)
    nmatd = numpy.zeros([rows-(flags+fleads),cols*flags])
    # Do this only if leads are also required !
    if fleads != None:
        nmatdf = numpy.zeros([rows-(flags+fleads),cols*fleads])
        for lead in range(0,fleads,1):
            for col in range(0,cols,1):
                shift = lead*(cols-1)
                if lead == 0:
                    nmatdf[:,col+shift+lead] = matd[flags+lead:-fleads,col]
                elif lead == fleads - 1:
                    nmatdf[:,col+shift+lead] = matd[flags+lead:-1,col]
                elif lead > 0:
                    nmatdf[:,col+shift+lead] = matd[flags+lead:-(fleads-lead),col]
    for lag in range(0,flags,1):
        for col in range(0,cols,1):
            shift = lag*(cols-1)
            if lag == 0:
                nmatd[:,col+shift+lag] = matd[lag:-(fleads+flags-lag),col]
            elif lag == flags - 1:
                nmatd[:,col+shift+lag] = matd[lag:-(1+fleads),col]
            elif lag > 0:
                nmatd[:,col+shift+lag] = matd[lag:-(fleads+flags-lag),col]
    # Flip matrix left to right in order to have p(1) lags first, then re-order variables
    nmatd = numpy.fliplr(nmatd)
    for elem in range(0,flags,1):
        nmatd[:,elem*cols:(elem+1)*cols] = numpy.fliplr(nmatd[:,elem*cols:(elem+1)*cols])
    # Add original data back to lags in first columns to get stacked X with current and lagged
    nmatd = numpy.hstack((matd[flags:-fleads,:],nmatd))
    # Stack also the leading terms if wanted !
    if fleads != None:
        nmatd = numpy.hstack((nmatd,nmatdf))
    # Also make a dictionary for each variable
    stackdic = {}
    for namo in vnames:
        tmpmat = numpy.zeros((rows-flags-fleads,flags+1+fleads))
        # Save the current-period value
        tmpmat[:,flags+1] = data[flags:fleads,vnames.index(namo)]
        for i1,lago in enumerate(xrange(flags,0,1)):
            tmpmat[:,i1] = nmatd[:,:]
            
    if not func:
        self.stackmat = copy.deepcopy(nmatd)
        return self
    elif func:
        return nmatd



def genEE(self,vnames=None,ydata=None,xdata=None,betta=None,hweightm=None,mlags=None,func=False):
    '''
    Function to generate equations residuals after controlling for own lags, used to extract factors
    Can also be used to estimate bettas, also optionally uses weighting matrix hweightm
    '''
    data = ydata
    cols = data.shape[1]
    nlags = self.nlags
    flags = self.flags
    npc = self.sfacs
    sfacs = npc
    if vnames == None:
        vnames = self.vnames
    auto_betta = {}
    auto_ee_dic = {}
    lxmatdic = xdata
    for varo in range(0,cols,1):
        Xmat = lxmatdic[vnames[varo]]
        Ymat = data[:,varo]
        row_min = min(Xmat.shape[0],Ymat.shape[0])
        Xmat_indi = Xmat.shape[0] - row_min
        Ymat_indi = Ymat.shape[0] - row_min
        Ymat = Ymat[Ymat_indi:]
        Xmat = Xmat[Xmat_indi:,:]
        if betta == None:
            if hweightm == None:
                XX = numpy.dot(Xmat.T,Xmat)
            else:
                XTW = numpy.dot(Xmat.T,hweightm)
                XX = numpy.dot(XTW,Xmat)
            XXi = numpy.linalg.inv(XX)
            if hweightm == None:
                Xy = numpy.dot(Xmat.T,Ymat)
            else:
                XTW = numpy.dot(Xmat.T,hweightm)
                Xy = numpy.dot(XTW,Ymat)
            auto_betta[vnames[varo]] = numpy.dot(XXi,Xy)
        if betta == None:
            auto_ee_dic[vnames[varo]] = Ymat - numpy.dot(Xmat,auto_betta[vnames[varo]])
        else:
            auto_ee_dic[vnames[varo]] = Ymat - numpy.dot(Xmat,betta[vnames[varo]])
            auto_betta = copy.deepcopy(betta)

    ee_mat = auto_ee_dic[vnames[0]]
    for varo in range(1,cols,1):
        ee_mat = numpy.vstack((ee_mat,auto_ee_dic[vnames[varo]]))
    ee_mat = ee_mat.T     
    if not func:
        self.auto_betta = copy.deepcopy(auto_betta)
        self.auto_ee_dic = copy.deepcopy(auto_ee_dic)
        self.ee_mat = copy.deepcopy(ee_mat)
        return self
    elif func:
        return auto_betta,auto_ee_dic,ee_mat


def genLX(self,vnames=None,ydata=None,nlags=None,mlags=None,func=False):
    '''
    Function to create a dictionary of the auto-regressors for each variable to eliminate serial corr
    so be careful, this will only create the regressor matrices involving OWN lags
    '''
    # Get all the needed variables from instance
    if nlags == None:
        nlags = copy.deepcopy(self.nlags)
    if mlags == None:
        mlags = nlags
    flags = copy.deepcopy(self.flags)
    if ydata == None:
        matd = copy.deepcopy(self.tdata)
    else:
        matd = ydata
    rows = matd.shape[0]
    cols = matd.shape[1]
    if vnames == None:
        vnames = copy.deepcopy(self.vnames)
    lxmatdic = {}
    for col in range(0,cols,1):
        nmatd = numpy.zeros([rows-(mlags),nlags])
        for lag in range(0,nlags,1):
            if lag == 0:
                nmatd[:,lag] = matd[lag:-((mlags)-lag),col]
            elif lag == nlags - 1:
                nmatd[:,lag] = matd[(mlags-1):-1,col]
            elif lag > 0:
                nmatd[:,lag] = matd[lag:-((mlags)-lag),col]
            # Flip matrix left to right in order to have p(1) lags first
        nmatd = numpy.fliplr(nmatd)
        lxmatdic[vnames[col]] = copy.deepcopy(nmatd)
    if not func:
        self.lxmatdic = copy.deepcopy(lxmatdic)
        return self
    elif func:
        return lxmatdic



__all__ = ['DIV']

from .base import NaryHead
from ..core import init_module

init_module.import_heads()

class DivHead(NaryHead):

    """
    DivHead represents division n-ary operation,
    data is a n-tuple of expression operands.
    """
    op_mth = '__div__'
    op_rmth = '__rdiv__'
    op_symbol = '/'


    def __repr__(self): return 'DIV'

    def new(self, cls, operands, evaluate=True):
        m = len(operands)
        if m==0:
            return cls(NUMBER, 1)
        if m==1:
            return operands[0]
        if not evaluate:
            return cls(self, operands)
        return operands[0] / MUL.new(cls, reversed(operands[1:]))

    def reevaluate(self, cls, operands):
        r = operands[0] if operands else cls(NUMBER, 1)
        for op in operands[1:]:
            r /= op
        return r

    def term_coeff(self, cls, expr):
        return expr, 1

    def commutative_mul(self, cls, lhs, rhs):
        d = {}
        imul = BASE_EXP_DICT.inplace_commutative_data_mul
        for i, op in enumerate(lhs.data):
            if i:
                imul(cls, d, 1/op)
            else:
                imul(cls, d, op)
        imul(cls, d, rhs)
        return BASE_EXP_DICT.new(cls, d)

    def pow(self, cls, lhs, rhs):
        data = lhs.data
        l = [data[0]] + [1/d for d in data[1:]]
        return cls(MUL, l) ** rhs

    def algebra_add_number(self, Algebra, lhs, rhs, inplace):
        if not rhs:
            return lhs
        return Algebra(ADD, [lhs, Algebra(NUMBER, rhs)])
        
DIV = DivHead()




__all__ = ['NEG']

from .base import heads_precedence, ArithmeticHead
from ..core import init_module

init_module.import_heads()
init_module.import_lowlevel_operations()
init_module.import_numbers()

class NegHead(ArithmeticHead):

    """
    NegHead represents negative unary operation where operand (data)
    is an expression.
    """

    op_mth = '__neg__'

    def is_data_ok(self, cls, data):
        if not isinstance(data, cls):
            return '%s data part must be %s instance but got %s' % (self, cls, type(data))
        
    def __repr__(self): return 'NEG'

    def new(self, cls, expr, evaluate=True):
        if not evaluate:
            return cls(self, expr)
        h, d = expr.pair
        if h is NUMBER:
            return cls(NUMBER, -d)
        if h is TERM_COEFF:
            t, c = d
            return TERM_COEFF.new(cls, (t, -c))
        return cls(NEG, expr)

    def reevaluate(self, cls, expr):
        return -expr

    def data_to_str_and_precedence(self, cls, expr):
        if cls.algebra_options.get('evaluate_addition'):
            if expr.head is NEG:
                expr = expr.data
                return expr.head.data_to_str_and_precedence(cls, expr.data)
        s, s_p = expr.head.data_to_str_and_precedence(cls, expr.data)
        neg_p = heads_precedence.NEG
        if s_p < neg_p:
            return '-(' + s + ')', neg_p
        return '-' + s, neg_p

    def to_lowlevel(self, cls, data, pair):
        if isinstance(data, numbertypes):
            return -data
        if data.head is NUMBER:
            return -data.data
        return cls(TERM_COEFF, (data, -1))

    def term_coeff(self, cls, expr):
        e = expr.data
        t, c = e.head.term_coeff(cls, e)
        return t, -c

    def scan(self, proc, cls, expr, target):
        expr.head.scan(proc, cls, expr.data, target)
        proc(cls, self, expr, target)

    def walk(self, func, cls, operand, target):
        operand1 = operand.head.walk(func, cls, operand.data, operand)
        if operand1 is operand:
            return func(cls, self, operand, target)
        r = self.new(cls, operand1)
        return func(cls, r.head, r.data, r)

    def to_TERM_COEFF_DICT(self, Algebra, data, expr):
        return -data.head.to_TERM_COEFF_DICT(Algebra, data.data, data)

    def to_ADD(self, Algebra, data, expr):
        return -data.head.to_ADD(Algebra, data.data, data)

    def algebra_pos(self, Algebra, expr):
        return expr

    def algebra_neg(self, Algebra, expr):
        if Algebra.algebra_options.get('evaluate_addition'):
            return expr.data
        return Algebra(NEG, expr)

    def algebra_add_number(self, Algebra, lhs, rhs, inplace):
        return self.algebra_add(Algebra, lhs, Algebra(NUMBER, rhs), inplace)

    def algebra_add(self, Algebra, lhs, rhs, inplace):
        rhead, rdata = rhs.pair
        if rhead is ADD:
            data = [lhs] + rdata
        elif rhead is TERM_COEFF_DICT or rhead is EXP_COEFF_DICT:
            data = [lhs] + rhs.to(ADD).data
        else:
            data = [lhs, rhs]
        if Algebra.algebra_options.get('evaluate_addition'):
            ADD.combine_add_list(Algebra, data)
        return add_new(Algebra, data)

    def algebra_mul_number(self, Algebra, lhs, rhs, inplace):
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            term, coeff = lhs.head.term_coeff(Algebra, lhs)
            return term_coeff_new(Algebra, (term, coeff * rhs))
        else:
            if Algebra.algebra_options.get('evaluate_addition'):
                if rhs == 0:
                    return Algebra(NUMBER, 0)
                term, coeff = lhs.head.term_coeff(Algebra, lhs)
                return term_coeff_new(Algebra, (term, coeff * rhs))
            return mul_new(Algebra, [lhs, Algebra(NUMBER, rhs)])

    def algebra_mul(self, Algebra, lhs, rhs, inplace):
        ldata = lhs.data
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            return super(type(self), self).algebra_mul(Algebra, lhs, rhs, inplace)
        else:
            if Algebra.algebra_options.get('evaluate_addition'):
                rhead, rdata = rhs.pair
                if rhead is NUMBER:
                    return ldata.head.algebra_mul_number(Algebra, ldata, -rdata, inplace)
                return super(type(self), self).algebra_mul(Algebra, lhs, rhs, inplace)
            return mul_new(Algebra, [lhs, rhs])
    
NEG = NegHead()



# Parallel Python Software: http://www.parallelpython.com
# Copyright (c) 2005-2012, Vitalii Vanovschi
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of the author nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
"""
Parallel Python Software, PP Transport

http://www.parallelpython.com - updates, documentation, examples and support
forums
"""

import logging
import os
import socket
import struct
import types

copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
version = "1.6.2"


# compartibility with Python 2.6
try:
    import hashlib
    sha_new = hashlib.sha1
    md5_new = hashlib.md5
except ImportError:
    import sha
    import md5
    sha_new = sha.new
    md5_new = md5.new


class Transport(object):

    def send(self, msg):
        raise NotImplemented("abstact function 'send' must be implemented "\
                "in a subclass")

    def receive(self, preprocess=None):
        raise NotImplemented("abstact function 'receive' must be implemented "\
                "in a subclass")

    def authenticate(self, secret):
        remote_version = self.receive()
        if version != remote_version:
            logging.error("PP version mismatch (local: pp-%s, remote: pp-%s)"
                % (version, remote_version))
            logging.error("Please install the same version of PP on all nodes")
            return False
        srandom = self.receive()
        answer = sha_new(srandom+secret).hexdigest()
        self.send(answer)
        response = self.receive()
        if response == "OK":
            return True
        else:
            return False

    def close(self):
        pass

    def _connect(self, host, port):
        pass


class CTransport(Transport):
    """Cached transport
    """
    rcache = {}

    def hash(self, msg):
        return md5_new(msg).hexdigest()

    def csend(self, msg):
        hash1 = self.hash(msg)
        if hash1 in self.scache:
            self.send("H" + hash1)
        else:
            self.send("N" + msg)
            self.scache[hash1] = True

    def creceive(self, preprocess=None):
        msg = self.receive()
        if msg[0] == 'H':
            hash1 = msg[1:]
        else:
            msg = msg[1:]
            hash1 = self.hash(msg)
            self.rcache[hash1] = map(preprocess, (msg, ))[0]
        return self.rcache[hash1]


class PipeTransport(Transport):

    def __init__(self, r, w):
        self.scache = {}
        self.exiting = False
        if isinstance(r, types.FileType) and isinstance(w, types.FileType):
            self.r = r
            self.w = w
        else:
            raise TypeError("Both arguments of PipeTransport constructor " \
                    "must be file objects")

    def send(self, msg):
        self.w.write(struct.pack("!Q", len(msg)))
        self.w.flush()
        self.w.write(msg)
        self.w.flush()

    def receive(self, preprocess=None):
        e_size = struct.calcsize("!Q")
        r_size = 0
        data = ""
        while r_size < e_size:
            msg = self.r.read(e_size-r_size)
            if msg == "":
                raise RuntimeError("Communication pipe read error")
            r_size += len(msg)
            data += msg
        e_size = struct.unpack("!Q", data)[0]

        r_size = 0
        data = ""
        while r_size < e_size:
            msg = self.r.read(e_size-r_size)
            if msg == "":
                raise RuntimeError("Communication pipe read error")
            r_size += len(msg)
            data += msg

        return map(preprocess, (data, ))[0]

    def close(self):
        self.w.close()
        self.r.close()


class SocketTransport(Transport):

    def __init__(self, socket1, socket_timeout):
        if socket1:
            self.socket = socket1
        else:
            self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.settimeout(socket_timeout)
        self.scache = {}

    def send(self, data):
        size = struct.pack("!Q", len(data))
        t_size = struct.calcsize("!Q")
        s_size = 0L
        while s_size < t_size:
            p_size = self.socket.send(size[s_size:])
            if p_size == 0:
                raise RuntimeError("Socket connection is broken")
            s_size += p_size

        t_size = len(data)
        s_size = 0L
        while s_size < t_size:
            p_size = self.socket.send(data[s_size:])
            if p_size == 0:
                raise RuntimeError("Socket connection is broken")
            s_size += p_size

    def receive(self, preprocess=None):
        e_size = struct.calcsize("!Q")
        r_size = 0
        data = ""
        while r_size < e_size:
            msg = self.socket.recv(e_size-r_size)
            if msg == "":
                raise RuntimeError("Socket connection is broken")
            r_size += len(msg)
            data += msg
        e_size = struct.unpack("!Q", data)[0]

        r_size = 0
        data = ""
        while r_size < e_size:
            msg = self.socket.recv(e_size-r_size)
            if msg == "":
                raise RuntimeError("Socket connection is broken")
            r_size += len(msg)
            data += msg
        return data

    def close(self):
        self.socket.close()

    def _connect(self, host, port):
        self.socket.connect((host, port))


class CPipeTransport(PipeTransport, CTransport):
    pass


class CSocketTransport(SocketTransport, CTransport):
    pass
    
# Parallel Python Software: http://www.parallelpython.com



__all__ = ['SUB']

from .base import heads_precedence, ArithmeticHead

class SubHead(ArithmeticHead):

    """
    SubHead represents subtraction n-ary operation where operands is
    given as a n-tuple of expressions.
    """
    op_mth = '__sub__'
    op_rmth = '__rsub__'

    def new(self, cls, operands, evaluate=True):
        n = len(operands)
        if n==1:
            return operands[0]
        if n==0:
            return cls(NUMBER, 0)
        return cls(self, operands)

    def __repr__(self): return 'SUB'

    def reevaluate(self, cls, operands):
        r = operands[0] if operands else cls(NUMBER, 0)
        for op in operands[1:]:
            r -= op
        return r
    
    def data_to_str_and_precedence(self, cls, operands):
        m = len(operands)
        if m==0:
            return '0', heads_precedence.NUMBER
        if m==1:
            op = operands[0]
            return op.head.data_to_str_and_precedence(cls, op.data)
        sub_p = heads_precedence.SUB
        r = ''
        for op in operands:
            t,t_p = op.head.data_to_str_and_precedence(cls, op.data)
            if not r:
                r += '(' + t + ')' if t_p < sub_p else t
            elif t.startswith('-') and t_p > sub_p:
                r += ' + ' + t[1:]
            else:
                r += ' - (' + t + ')' if t_p <= sub_p else ' - ' + t
        return r, sub_p

    def walk(self, func, cls, data, target):
        l = []
        flag = False
        for op in data:
            o = op.head.walk(func, cls, op.data, op)
            if op is not o:
                flag = True
            l.append(o)
        if flag:
            r = SUB.new(cls, l)
            return func(cls, r.head, r.data, r)
        return func(cls, self, data, target)

SUB = SubHead()




def monitor(f, input='print', output='print'):
    """
    Returns a wrapped copy of *f* that monitors evaluation by calling
    *input* with every input (*args*, *kwargs*) passed to *f* and
    *output* with every value returned from *f*. The default action
    (specify using the special string value ``'print'``) is to print
    inputs and outputs to stdout, along with the total evaluation
    count::

        >>> from mpmath import *
        >>> mp.dps = 5; mp.pretty = False
        >>> diff(monitor(exp), 1)   # diff will eval f(x-h) and f(x+h)
        in  0 (mpf('0.99999999906867742538452148'),) {}
        out 0 mpf('2.7182818259274480055282064')
        in  1 (mpf('1.0000000009313225746154785'),) {}
        out 1 mpf('2.7182818309906424675501024')
        mpf('2.7182808')

    To disable either the input or the output handler, you may
    pass *None* as argument.

    Custom input and output handlers may be used e.g. to store
    results for later analysis::

        >>> mp.dps = 15
        >>> input = []
        >>> output = []
        >>> findroot(monitor(sin, input.append, output.append), 3.0)
        mpf('3.1415926535897932')
        >>> len(input)  # Count number of evaluations
        9
        >>> print(input[3]); print(output[3])
        ((mpf('3.1415076583334066'),), {})
        8.49952562843408e-5
        >>> print(input[4]); print(output[4])
        ((mpf('3.1415928201669122'),), {})
        -1.66577118985331e-7

    """
    if not input:
        input = lambda v: None
    elif input == 'print':
        incount = [0]
        def input(value):
            args, kwargs = value
            print("in  %s %r %r" % (incount[0], args, kwargs))
            incount[0] += 1
    if not output:
        output = lambda v: None
    elif output == 'print':
        outcount = [0]
        def output(value):
            print("out %s %r" % (outcount[0], value))
            outcount[0] += 1
    def f_monitored(*args, **kwargs):
        input((args, kwargs))
        v = f(*args, **kwargs)
        output(v)
        return v
    return f_monitored

def timing(f, *args, **kwargs):
    """
    Returns time elapsed for evaluating ``f()``. Optionally arguments
    may be passed to time the execution of ``f(*args, **kwargs)``.

    If the first call is very quick, ``f`` is called
    repeatedly and the best time is returned.
    """
    once = kwargs.get('once')
    if 'once' in kwargs:
        del kwargs['once']
    if args or kwargs:
        if len(args) == 1 and not kwargs:
            arg = args[0]
            g = lambda: f(arg)
        else:
            g = lambda: f(*args, **kwargs)
    else:
        g = f
    from timeit import default_timer as clock
    t1=clock(); v=g(); t2=clock(); t=t2-t1
    if t > 0.05 or once:
        return t
    for i in range(3):
        t1=clock();
        # Evaluate multiple times because the timer function
        # has a significant overhead
        g();g();g();g();g();g();g();g();g();g()
        t2=clock()
        t=min(t,(t2-t1)/10)
    return t
